//Jamie Morken April 22/2003	Rocket Research

#include <io.h>
#include <interrupt.h>
#include <signal.h>
#include <progmem.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <eeprom.h>
#include "printf.h"
#include "uart.h"


#define t1 72	//value/3600 is how often task will be called in seconds

int time1;

void task1(void);

unsigned char r_char;  		//current character we are receiving from the UART
volatile u08 data;		//variable for using USART to communicate with the PC, stores value of received character
volatile u08 CharAvail;	//variable for using USART to communicate with the PC, indicates if a character is available

volatile u16 mod3;		//timer variable


SIGNAL(SIG_UART_RECV)    //this signal used to be handled in uart.c until I moved it here for easier use  
{
    //Store the received character
	r_char = inp(UDR);

	data = r_char;
	CharAvail = 1;

}	//end of SIGNAL(SIG_UART_RECV)


SIGNAL(SIG_OVERFLOW2)	//overflows 3600 times/second at 7372800Hz
{   

	// Decrement the task timers if they are not already one (zero is reservered so that a task can be shut off)
	if (time1>1)	--time1;



	mod3++; //increment our 16 bit variable everytime OCR2 = TCNT2 (once every 128 clock pulses to TCNT2)

	if (mod3 == 3600) 
	{ 

		mod3 = 1;

	} 


}	// end of SIGNAL(SIG_OVERFLOW0)


void OneMsDelay(void)
{
u16 tmp=(mod3+3)%3600;	//0.83ms  mod3+3 at (mod++ = 1/3600th of a second) gives (1/3600)*3 as the delay
while (tmp!=mod3); 
}

void TenMsDelay(void) 
{ 
u16 tmp=(mod3+36)%3600; 
while (tmp!=mod3); 
} 

void OneHundredMsDelay(void) 
{ 
u16 tmp=(mod3+360)%3600; 
while (tmp!=mod3); 
} 

void OneSecondDelay(void) 
{ 
u16 tmp=(mod3+3599)%3600; 
while (tmp!=mod3); 
} 

void initialize(void)
{
  
	//initialize task timers
	time1=t1;

	//set up 8 bit timer 2  (simple task scheduler) 
	TCNT2=0;			// preload the timer so that it interrupts after 0.2778ms (this is 3600times/sec - gives 3X sampling of 1200 BAUD radio)
	TCCR2=2;			// prescalar to 8 - if prescaler 8 then 8 clk's gives 1 count to TCNT0  -  7372800/3600 = 2048 clocks per bit at 3600baud
	TIMSK=64;			// turn on the timer overflow ISR  			// turn on timer 2 overflow ISR  

	sbi(DDRD, PD6);		//set mosfet channel 1 (TP1 on board) to output (propellant valve 1)
	sbi(DDRD, PD7);		//set mosfet channel 2 (TP2 on board) to output (propellant valve 2)
	sbi(DDRB, PB2);		//set mosfet channel 3 (TP3 on board) to output (ignitor coil)

}

int main(void)
{				
	
	UART_Init();
	initialize();


	PRINT("rocket ignitor tester V 1.0\r\n");

	while (1)
	{
	    if (time1==1)	task1();

	}

}	//end of main


void task1(void)
{       

	time1=t1;

	int t1multiVar;

	if (CharAvail == 1)
		{ 

		CharAvail = 0;

		switch (data) 
			{ 
	
			
			case '-':						// pulse propellant valve 1 continously at 60ms interval
				{
					cbi(PORTD, PD6);

					for(t1multiVar = 0; t1multiVar < 40; t1multiVar++)
					{
						sbi(PORTD, PD6);		// turn on valve
						TenMsDelay();
						TenMsDelay();
						TenMsDelay();
						cbi(PORTD, PD6);		// turn off valve
						TenMsDelay();
						TenMsDelay();
						TenMsDelay();
					}

					PRINT("valve toggled 40 times");
					EOL();
					PRINT("\n\n");

				} 
			break; 


			case '=':						// pulse propellant valve 2 continously at 20ms interval
				{
					cbi(PORTD, PD7);

					for(t1multiVar = 0; t1multiVar < 40; t1multiVar++)
					{
						sbi(PORTD, PD7);		// turn on valve
						TenMsDelay();
						cbi(PORTD, PD7);		// turn off valve
						TenMsDelay();
					}

					PRINT("valve toggled 40 times");
					EOL();
					PRINT("\n\n");

				} 
			break; 
			

			case '0':						// pulse propellant valve 2 continously at 40ms interval
				{
					cbi(PORTD, PD7);

					for(t1multiVar = 0; t1multiVar < 40; t1multiVar++)
					{
						sbi(PORTD, PD7);		// turn on valve
						TenMsDelay();
						TenMsDelay();
						cbi(PORTD, PD7);		// turn off valve
						TenMsDelay();
						TenMsDelay();
					}

					PRINT("valve toggled 40 times");
					EOL();
					PRINT("\n\n");

				} 
			break; 


			case '1': 
				{ 
					if (bit_is_set(PORTD, PD6))	//toggle propellant valve 1
					{	
						cbi(PORTD, PD6);
						PRINT("propellant valve 1 off");
						EOL();
						PRINT("\n\n");
					}
					else
					{
						sbi(PORTD, PD6);
						PRINT("propellant valve 1 on");
						EOL();
						PRINT("\n\n");
					}

				} 
			break; 


			case '2': 
				{ 
					if (bit_is_set(PORTD, PD7))	//toggle propellant valve 2
					{	
						cbi(PORTD, PD7);
						PRINT("propellant valve 2 off");
						EOL();
						PRINT("\n\n");
					}
					else
					{
						sbi(PORTD, PD7);
						PRINT("propellant valve 2 on");
						EOL();
						PRINT("\n\n");
					}

				} 
			break; 


			case '3': 
				{ 
					if (bit_is_set(PORTB, PB2))	//toggle ignitor coil
					{	
						cbi(PORTB, PB2);
						PRINT("ignitor coil off");
						EOL();
						PRINT("\n\n");
					}
					else
					{
						sbi(PORTB, PB2);
						PRINT("ignitor coil on");
						EOL();
						PRINT("\n\n");
					}

				} 
			break; 


			case '4':						// pulse ignitor coil 1ms
				{
					cbi(PORTB, PB2);
					OneSecondDelay();		// make sure coil is de-energized before we pulse it

					sbi(PORTB, PB2);		// energize coil
					OneMsDelay();
					cbi(PORTB, PB2);		// de-energize coil (spark occurs now)
					
					PRINT("coil sparked");
					EOL();
					PRINT("\n\n");

				} 
			break; 


			case '5':						// pulse ignitor coil 5ms
				{
					cbi(PORTB, PB2);
					OneSecondDelay();		// make sure coil is de-energized before we pulse it

					sbi(PORTB, PB2);		// energize coil
					OneMsDelay();
					OneMsDelay();
					OneMsDelay();
					OneMsDelay();
					OneMsDelay();
					cbi(PORTB, PB2);		// de-energize coil (spark occurs now)
					
					PRINT("coil sparked");
					EOL();
					PRINT("\n\n");

				} 
			break; 

			
			case '6':						// pulse ignitor coil 10ms
				{
					cbi(PORTB, PB2);
					OneSecondDelay();		// make sure coil is de-energized before we pulse it

					sbi(PORTB, PB2);		// energize coil
					TenMsDelay();
					cbi(PORTB, PB2);		// de-energize coil (spark occurs now)
					
					PRINT("coil sparked");
					EOL();
					PRINT("\n\n");

				} 
			break; 

			
			case '7':						// pulse ignitor coil continously at 6ms interval
				{
					cbi(PORTB, PB2);

					for(t1multiVar = 0; t1multiVar < 500; t1multiVar++)
					{
						sbi(PORTB, PB2);		// energize coil
						OneMsDelay();
						OneMsDelay();
						OneMsDelay();
						cbi(PORTB, PB2);		// de-energize coil (spark occurs now)
						OneMsDelay();
						OneMsDelay();
						OneMsDelay();
					}

					PRINT("coil sparked 500 times");
					EOL();
					PRINT("\n\n");

				} 
			break; 

			
			case '8':						// pulse ignitor coil continously at 10ms interval
				{
					cbi(PORTB, PB2);

					for(t1multiVar = 0; t1multiVar < 250; t1multiVar++)
					{
						sbi(PORTB, PB2);		// energize coil
						OneMsDelay();
						OneMsDelay();
						OneMsDelay();
						OneMsDelay();		
						OneMsDelay();
						cbi(PORTB, PB2);		// de-energize coil (spark occurs now)
						OneMsDelay();
						OneMsDelay();
						OneMsDelay();
						OneMsDelay();		
						OneMsDelay();
					}

					PRINT("coil sparked 250 times");
					EOL();
					PRINT("\n\n");

				} 
			break; 


			case '9': 
				{ 
				
					cbi(PORTD, PD6);
					cbi(PORTD, PD7);
					cbi(PORTB, PB2);
					PRINT("everything turned off");
					EOL();
					PRINT("\n\n");

				} 
			break; 

			case '?':
			default:
  	  
				PRINT("\n-....pulse propellant valve 1 continously at 60ms interval 1\r");
				PRINT("\n=....pulse propellant valve 2 continously at 20ms interval 1\r");
				PRINT("\n0....pulse propellant valve 2 continously at 40ms interval 1\r");
				PRINT("\n1....toggle propellant valve 1\r");
				PRINT("\n2....toggle propellant valve 2\r");
				PRINT("\n3....toggle ignitor coil\r");
				PRINT("\n4....pulse ignitor coil 1ms\r");
				PRINT("\n5....pulse ignitor coil 5ms\r");
				PRINT("\n6....pulse ignitor coil 10ms\r");
				PRINT("\n7....pulse ignitor coil continously at 6ms interval\r");
				PRINT("\n8....pulse ignitor coil continously at 10ms interval\r");
				PRINT("\n9....turn off everything\r");
				PRINT("\n?....Print Help\r\n\n"); 

			break;

			}	//Switch

		}	//If 

}	// end of task1



