//Jamie Morken May 10/2000	Rocket Research


#include <io.h>
#include <interrupt.h>
#include <signal.h>
#include <progmem.h>
#include "uart.h"

volatile u08 desiredServoOnTime; //somewhere between 100 for 1ms and 200 for 2ms at 100kHz 
volatile u08 actualServoOnTime; 
volatile u16 desiredWaitPeriod; //time between servoOn's, specced as 20ms so at 100kHz it is 2000
volatile u16 actualWaitPeriod; 
volatile u08 waiting;	//1 if we are in a 20ms waitperiod, 0 if we are sending the servo control line high
volatile u08 servoOff;	//1 if we have turned the servo off, 0 if the servo is on
volatile u08 giveUartAChance;	//1 if we disrupt our counter to wait for Uart command, 0 if we are still controlling the servo
volatile u16 desiredLoopCount;	//how many counter loops we do before waiting for uart
volatile u16 actualLoopCount;
volatile u08 continousControl;	//if 1 then hold servo at desired angle, if 0 turn servo off after it has been moved towards the desired location

SIGNAL(SIG_OUTPUT_COMPARE2) //called whenever OCR2 = TCNT2
{ 
//frequency we are here = (PCK/outputcompare) = (3686400/36) = every 1/102400 of a second


outp(0,TCNT2); //reset the Timer/Counter2 output compare register to zero - can do this auto with TCCR2 register..

//cbi(PORTD, PD4); turns servo voltage to 0V
//sbi(PORTD, PD4); turns servo voltage to 5V

if(actualLoopCount < desiredLoopCount)	//if we are still commanding the servo then we can't access the UART for some reason so I put this here to stop commanding the servo every desiredLoopCount times we are here
{
//a desiredLoopCount of 20000 should disable the timer/counter every 200ms and therefore give the UART a chance to send a new command
actualLoopCount++;

if (waiting == 0)	//we aren't waiting we are controlling the servo line high
{
	if(actualServoOnTime < desiredServoOnTime)
	{
		actualServoOnTime++;
		sbi(PORTD, PD4);
	}

	else	//the servo control line has been high for the required time so now set it to low for 20ms 
	{
		actualServoOnTime = 0;	//reset actualServoOnTime for next time
		cbi(PORTD, PD4);
		waiting = 1;	//we are now in a wait period for 20ms
	}

}//end of if(waiting == 0)

else	//waiting == 1 so we are in a 20ms period or the servo is off
{
	if(servoOff == 1)	//if the servo is off we do nothing
	{}

	else	//the servo is on but we are in a 20ms wait period
	{
		if(actualWaitPeriod < desiredWaitPeriod)	//we are still in wait period..
		{
			actualWaitPeriod++;	//show that we have been waiting..
			cbi(PORTD, PD4);
		}

		else	//we are done the wait period so we can turn the servo control line back on
		{
			actualWaitPeriod = 0;	//reset actualWaitPeriod for next time
			sbi(PORTD, PD4);	//set the servo control line to high
			waiting = 0;	//indicate we are done waiting

		}

	}	//end of elseif(servoOff == 1)

}//end of elseif(waiting == 0)

}//end of if(actualLoopCount < desiredLoopCount)

else
{

actualLoopCount = 0; //reset our actualLoopCount as we are now done the servo control and are going to give the uart a chance to communicate
cbi(TIMSK,7); //disable interrupts for Timer/Counter2	//reenabled once UART has a chance to send us something..

}//end of elseif(actualLoopCount < desiredLoopCount)

} //end of SIGNAL(SIG_OUTPUT_COMPARE2) 
 
 
void INIT_TNCTimer(void) 
{ 
 outp(0,ASSR); 	//set Timer/Counter2 to be clocked from the system clock CLK 
 outp(0x01,TCCR2); //select Timer/Counter2's clock prescaler PCK2, for 0x01 it is CLK
 outp(0,TCNT2); //initialize Timer/Counter2's value to zero 
 outp(0x24,OCR2); //put initial value in Timer2's output compare register - generates int when ==
 //0x24 is 36 in decimal clk/36=~100kHz
 sbi(TIMSK,7); //enable interrupts for Timer/Counter2 (pg24)
 
} //end of void INIT_TNCTimer(void) 
 


int main(void)
{


u08 i;
u08 data; 

INIT_TNCTimer(); 
UART_Init();


sbi(DDRD, PD4);		//set PD4 (Servo1) pin to output

desiredServoOnTime = 0;	//time when servo control voltage is 5V (specced between 1 and 2ms)
actualServoOnTime = 0;
desiredWaitPeriod = 2000;	//time when servo control voltage is 0V (specced at 20ms)
actualWaitPeriod = 0;
waiting = 1;		
servoOff = 1;
desiredLoopCount = 20000;	//how many counter loops we do before waiting for uart (20000 is 200ms)
actualLoopCount = 0;
continousControl = 1;	//we default to continous control of the servo

PRINT("R/C servo control testing V 0.1\r\n");

while(1)
{
if (UART_CharAvail())
 {
 data = UART_ReceiveByte();
 switch (data)
  {

  case 'z':
  	   {
	     	//set PD4 (Servo1) pin to high (2ms on time) counterclockwise from top of servo
		outp(0,TCNT2);
		sbi(TIMSK,7); 
		actualServoOnTime = 0;
		desiredServoOnTime = 95;
		actualWaitPeriod = 0;
		waiting = 0;
		servoOff = 0;	//indicate that the servo is on (whether its control line is high or not)
	      PRINT("servo1 high 2ms");
	      EOL();
	    PRINT("\n\n");
	   }
	   break;

	   
  case 'x': 
  	   { 
		   //set PD4 (Servo1) pin to middle (1.5ms on time)
		outp(0,TCNT2);
		sbi(TIMSK,7); 
		actualServoOnTime = 0;
		desiredServoOnTime = 55;
		actualWaitPeriod = 0;
		waiting = 0;
		servoOff = 0;
	      PRINT("servo1 middle 1.5ms"); 
	      EOL(); 
	    PRINT("\n\n"); 
	   } 
	   break; 

  case 'c': 
  	   { 
	     	//set PD4 (Servo1) pin to low (1ms on time) clockwise from top of servo
		outp(0,TCNT2);
		sbi(TIMSK,7); 
		actualServoOnTime = 0;
		desiredServoOnTime = 15;
		actualWaitPeriod = 0;
		waiting = 0;
		servoOff = 0;
	      PRINT("servo1 low 1ms"); 
	      EOL(); 
	    PRINT("\n\n"); 
	   } 
	   break; 


  case 'p': 
  	   { 
			//set PD4 (Servo1) pin to off (0ms on time)
		  outp(0,TCNT2);
		  actualServoOnTime = 0;
		  desiredServoOnTime = 0;
		  actualWaitPeriod = 0;
		  waiting = 1;
		  servoOff = 1;
	      PRINT("servo1 off 0ms"); 
	      EOL(); 
	    PRINT("\n\n"); 
	   } 
	   break; 

  case '?':
  default:
  	  
	  PRINT("\n\nz....set servo1 to high 2ms\r\n");
	  PRINT("\n\nx....set servo1 to middle 1.5ms\r\n");
  	  PRINT("\n\nc....set servo1 to low 1ms\r\n"); 
	  PRINT("\n\np....turn servo1 off 0ms\r\n");
   	  PRINT("\n\n1....set servo1 to position 1\r\n");
	  PRINT("\n\n2....set servo1 to position 2\r\n");
	  PRINT("\n\n3....set servo1 to position 3\r\n");
	  PRINT("\n\n4....set servo1 to position 4\r\n");
  	  PRINT("\n\n?....Print Help\r\n\n\n");

  break;

  }//Case

 }//end of If(UART_CharAvail())

if (continousControl == 1)	//only turn the ISR for the servo control back on if we want to hold servo at the desired position
{
	sbi(TIMSK,7); //reenable interrupts for Timer/Counter2 now that UART had a chance to send
}

}
	//end of while(1)
}
	//end of main

